@@ -15,7 +15,6 @@
 #include "DocumentClassPtr.h"
 #include "FloatList.h"
 #include "FontInfo.h"
-#include "Layout.h"
 #include "LayoutEnums.h"
 #include "LayoutModuleList.h"
 
@@ -30,11 +29,14 @@
 #include <string>
 #include <vector>
 
+#ifdef ERROR
+#undef ERROR
+#endif
+
 namespace lyx {
 
 namespace support { class FileName; }
 
-class Counters;
 class FloatList;
 class Layout;
 class LayoutFile;
@@ -147,6 +149,7 @@ public:
 		BASECLASS, //>This is a base class, i.e., top-level layout file
 		MERGE, //>This is a file included in a layout file
 		MODULE, //>This is a layout module
+		CITE_ENGINE, //>This is a cite engine
 		VALIDATION //>We're just validating
 	};
 	/// return values for read()
@@ -167,7 +170,8 @@ public:
 	ReturnValues read(Lexer & lex, ReadType rt = BASECLASS);
 	/// validates the layout information passed in str
 	static ReturnValues validate(std::string const & str);
-	///
+	/// \return the conversion of \param str to the latest layout format
+	/// compatible with the lyx format.
 	static std::string convert(std::string const & str);
 
 	///////////////////////////////////////////////////////////////////
@@ -188,6 +192,8 @@ public:
 	///
 	std::string const & name() const { return name_; }
 	///
+	std::string const & path() const { return path_; }
+	///
 	std::string const & category() const { return category_; }
 	///
 	std::string const & description() const { return description_; }
@@ -199,6 +205,16 @@ public:
 	OutputType outputType() const { return outputType_; }
 	/// Can be latex, docbook ... (the name of a format)
 	std::string outputFormat() const { return outputFormat_; }
+	/// Does this class redefine the output format?
+	bool hasOutputFormat() const { return has_output_format_; }
+	/// Return the non-localised names for the toc types.
+	std::map<std::string, docstring> const &
+		outlinerNames() const { return outliner_names_; }
+	/// \returns Layout named \p name if it exists, otherwise 0
+	Layout const * getLayout(docstring const & name) const;
+	/// \returns Layout named \p name if it exists, otherwise 0
+	Layout * getLayout(docstring const & name);
+
 protected:
 	/// Protect construction
 	TextClass();
@@ -232,6 +248,8 @@ protected:
 	mutable LayoutList layoutlist_;
 	/// Layout file name
 	std::string name_;
+	/// Layout file path (empty for system layout files)
+	std::string path_;
 	/// Class category
 	std::string category_;
 	/// document class name
@@ -250,15 +268,27 @@ protected:
 	mutable std::string prerequisites_;
 	/// The possible cite engine types
 	std::string opt_enginetype_;
+	/// The cite framework (bibtex, biblatex)
+	std::string citeframework_;
 	///
 	std::string opt_fontsize_;
 	///
+	std::string opt_pagesize_;
+	///
 	std::string opt_pagestyle_;
 	/// Specific class options
 	std::string options_;
+	/// Format of the fontsize option
+	std::string fontsize_format_;
+	/// Default page size
+	std::string pagesize_;
+	/// Format of the papersize option
+	std::string pagesize_format_;
 	///
 	std::string pagestyle_;
 	///
+	std::string tablestyle_;
+	///
 	std::string class_header_;
 	///
 	docstring defaultlayout_;
@@ -271,12 +301,17 @@ protected:
 	docstring htmlpreamble_;
 	/// same, but specifically for CSS information
 	docstring htmlstyles_;
-	/// the paragraph style to use for TOCs, Bibliography, etc
+	/// the paragraph style to use for TOCs, bibliography, etc.
 	mutable docstring html_toc_section_;
+	/// root element when exporting as DocBook
+	std::string docbookroot_;
+	/// whether this root element does not accept text without a section (i.e. the first text that is met in LyX must
+	/// be considered as the abstract if this is true); this text must be output within <info> and <abstract>
+	bool docbookforceabstract_;
 	/// latex packages loaded by document class.
 	std::set<std::string> provides_;
 	/// latex packages requested by document class.
-	std::set<std::string> requires_;
+	std::set<std::string> required_;
 	///
 	std::map<std::string, std::string> package_options_;
 	/// default modules wanted by document class
@@ -297,6 +332,8 @@ protected:
 	OutputType outputType_;
 	/// Can be latex, docbook ... (the name of a format)
 	std::string outputFormat_;
+	/// Does this class redefine the output format?
+	bool has_output_format_;
 	/** Base font. The paragraph and layout fonts are resolved against
 	    this font. This has to be fully instantiated. Attributes
 	    FONT_INHERIT, FONT_IGNORE, and FONT_TOGGLE are
@@ -322,11 +359,21 @@ protected:
 	/// Citation macros
 	std::map<CiteEngineType, std::map<std::string, std::string> > cite_macros_;
 	/// The default BibTeX bibliography style file
-	std::string cite_default_biblio_style_;
+	std::map<std::string, std::string> cite_default_biblio_style_;
+	/// Citation command aliases
+	std::map<std::string, std::string> cite_command_aliases_;
+	/// The maximum number of citations before "et al."
+	size_t maxcitenames_;
 	/// Whether full author lists are supported
 	bool cite_full_author_list_;
 	/// The possible citation styles
 	std::map<CiteEngineType, std::vector<CitationStyle> > cite_styles_;
+	/// Class-added citation styles
+	std::map<CiteEngineType, std::vector<CitationStyle> > class_cite_styles_;
+	///
+	std::map<std::string, docstring> outliner_names_;
+	/// Does this class put the bibliography to toc by itself?
+	bool bibintoc_;
 private:
 	///////////////////////////////////////////////////////////////////
 	// helper routines for reading layout files
@@ -340,7 +387,7 @@ private:
 	/// Reads the layout file without running layout2layout.
 	ReturnValues readWithoutConv(support::FileName const & filename, ReadType rt);
 	/// \return true for success.
-	bool readStyle(Lexer &, Layout &) const;
+	bool readStyle(Lexer &, Layout &, ReadType) const;
 	///
 	void readOutputType(Lexer &);
 	///
@@ -354,11 +401,15 @@ private:
 	///
 	bool readFloat(Lexer &);
 	///
-	bool readCiteEngine(Lexer &);
+	std::vector<CitationStyle> const & getCiteStyles(CiteEngineType const &) const;
+	///
+	bool readCiteEngine(Lexer &, ReadType, bool const add = false);
 	///
 	int readCiteEngineType(Lexer &) const;
 	///
-	bool readCiteFormat(Lexer &);
+	bool readCiteFormat(Lexer &, ReadType);
+	///
+	bool readOutlinerName(Lexer &);
 };
 
 
@@ -369,12 +420,8 @@ private:
 /// In that regard, DocumentClass objects are "dynamic". But this is really
 /// an illusion, since DocumentClass objects are not (currently) changed
 /// when, say, a new Module is loaded. Rather, the old DocumentClass is
-/// discarded---actually, it's kept around in case something on the cut
+/// discarded---actually, it will be kept around if something on the cut
 /// stack needs it---and a new one is created from scratch.
-///
-/// In the main LyX code, DocumentClass objects are created only by
-/// DocumentClassBundle, for which see below.
-///
 class DocumentClass : public TextClass {
 public:
 	///
@@ -386,7 +433,7 @@ public:
 	/// \return true if there is a Layout with latexname lay
 	bool hasLaTeXLayout(std::string const & lay) const;
 	/// A DocumentClass nevers count as loaded, since it is dynamic
-	virtual bool loaded() const { return false; }
+	bool loaded() const override { return false; }
 	/// \return the layout object of an inset given by name. If the name
 	/// is not found as such, the part after the ':' is stripped off, and
 	/// searched again. In this way, an error fallback can be provided:
@@ -396,7 +443,7 @@ public:
 	/// happen).  -- Idea JMarc, comment MV
 	InsetLayout const & insetLayout(docstring const & name) const;
 	/// a plain inset layout for use as a default
-	static InsetLayout const & plainInsetLayout() { return plain_insetlayout_; }
+	static InsetLayout const & plainInsetLayout();
 	/// add a new layout \c name if it does not exist in layoutlist_
 	/// \return whether we had to add one.
 	bool addLayoutIfNeeded(docstring const & name) const;
@@ -413,23 +460,39 @@ public:
 	///
 	std::string const & opt_enginetype() const { return opt_enginetype_; }
 	///
+	std::string const & citeFramework() const { return citeframework_; }
+	///
 	std::string const & opt_fontsize() const { return opt_fontsize_; }
 	///
+	std::string const & opt_pagesize() const { return opt_pagesize_; }
+	///
 	std::string const & opt_pagestyle() const { return opt_pagestyle_; }
 	///
 	std::string const & options() const { return options_; }
 	///
 	std::string const & class_header() const { return class_header_; }
 	///
+	std::string const & fontsizeformat() const { return fontsize_format_; }
+	///
+	std::string const & pagesize() const { return pagesize_; }
+	///
+	std::string const & pagesizeformat() const { return pagesize_format_; }
+	///
 	std::string const & pagestyle() const { return pagestyle_; }
 	///
+	std::string const & tablestyle() const { return tablestyle_; }
+	///
 	docstring const & preamble() const { return preamble_; }
 	///
 	docstring const & htmlpreamble() const { return htmlpreamble_; }
 	///
 	docstring const & htmlstyles() const { return htmlstyles_; }
+	///
+	bool docbookforceabstract() const { return docbookforceabstract_; }
+	///
+	std::string const & docbookroot() const { return docbookroot_; }
 	/// Looks for the layout of "highest level", other than Part (or other
-	/// layouts with a negative toc number), for use in constructing TOCs and 
+	/// layouts with a negative toc number), for use in constructing TOCs and
 	/// similar information.
 	Layout const & getTOCLayout() const;
 	/// the paragraph style to use for TOCs, Bibliography, etc
@@ -438,7 +501,7 @@ public:
 	/// is this feature already provided by the class?
 	bool provides(std::string const & p) const;
 	/// features required by the class?
-	std::set<std::string> const & requires() const { return requires_; }
+	std::set<std::string> const & required() const { return required_; }
 	/// package options to write to LaTeX file
 	std::map<std::string, std::string> const & packageOptions() const
 		{ return package_options_; }
@@ -469,8 +532,9 @@ public:
 	/// returns true if the class has a ToC structure
 	bool hasTocLevels() const;
 	///
-	std::string const & getCiteFormat(CiteEngineType const & type,
-		std::string const & entry, std::string const & fallback = "") const;
+	std::string const getCiteFormat(CiteEngineType const & type,
+		std::string const & entry, bool const punct = true,
+		std::string const & fallback = "") const;
 	///
 	std::string const & getCiteMacro(CiteEngineType const & type,
 		std::string const & macro) const;
@@ -479,22 +543,28 @@ public:
 	///
 	std::vector<CitationStyle> const & citeStyles(CiteEngineType const &) const;
 	///
-	std::string const & defaultBiblioStyle() const { return cite_default_biblio_style_; }
+	std::map<std::string, std::string> const & defaultBiblioStyle() const
+	{ return cite_default_biblio_style_; }
 	///
-	bool const & fullAuthorList() const { return cite_full_author_list_; }
+	std::map<std::string, std::string> const & citeCommandAliases() const
+	{ return cite_command_aliases_; }
+	/// The maximum number of citations before "et al."
+	size_t max_citenames() const { return maxcitenames_; }
+	///
+	bool fullAuthorList() const { return cite_full_author_list_; }
+	///
+	bool bibInToc() const { return bibintoc_; }
 protected:
 	/// Constructs a DocumentClass based upon a LayoutFile.
 	DocumentClass(LayoutFile const & tc);
 	/// Needed in tex2lyx
 	DocumentClass() {}
 private:
-	/// The only class that can create a DocumentClass is
-	/// DocumentClassBundle, which calls the protected constructor.
+	/// The only way to make a DocumentClass is to call this function.
 	friend DocumentClassPtr
 		getDocumentClass(LayoutFile const &, LayoutModuleList const &,
-				 bool const clone);
-	///
-	static InsetLayout plain_insetlayout_;
+				 std::string const &,
+				 bool clone, bool internal);
 };
 
 
@@ -504,13 +574,17 @@ private:
 /// on the CutStack.
 DocumentClassPtr getDocumentClass(LayoutFile const & baseClass,
 			LayoutModuleList const & modlist,
-			bool const clone = false);
+			std::string const & cengine = std::string(),
+			bool clone = false, bool internal = false);
 
 /// convert page sides option to text 1 or 2
 std::ostream & operator<<(std::ostream & os, PageSides p);
 
 /// current format of layout files
 extern int const LAYOUT_FORMAT;
+/// layout format for the current lyx file format (usually equal to
+/// LAYOUT_FORMAT)
+extern int const LYXFILE_LAYOUT_FORMAT;
 
 
 } // namespace lyx
