@@ -1,17 +1,17 @@
 #include <config.h>
 
-#include "../Messages.h"
+#include "LyXRC.h"
+
+#include "support/Messages.h"
 
 using namespace std;
 
 namespace lyx {
-	// Dummy LyXRC support
-	class LyXRC { string icon_set; } lyxrc;
+	// Dummy verbose support
+	bool verbose = false;
 
-	// Dummy LyXAlignment support
-	enum LyXAlignment {
-		DUMMY
-	};
+	// Dummy LyXRC support
+	LyXRC lyxrc;
 
 	// Keep the linker happy on Windows
 	void lyx_exit(int) {}
@@ -30,9 +30,4 @@ namespace lyx {
 		return lyx_messages;
 	}
 
-	string alignmentToCSS(LyXAlignment)
-	{
-		return string();
-	}
-
-}
+} // namespace lyx
